import greenfoot.*;

/**
 * The actor which jumps on platforms.
 * <p>
 * It works with one instance variable: gravity. If that is positive,
 * you jump. Otherwise, you fall. You will accelerate while you're
 * falling.
 * <p>
 * If you use this class, please put my name in the credits (if you
 * have so) and put my name in the documentation:
 * 
 * @author Sven van Nigtevecht
 * @version 1.0.1
 */
public class Jumper extends Actor
{
    /** Acceleration for falling and jumping */
    private int gravity;
    
    /** Image for when you face left */
    private GreenfootImage leftImage;
    /** Image for when you face right */
    private GreenfootImage rightImage;
    
    /**
     * Create a new Jumper.
     */
    public Jumper()
    {
        // the standard wombat image is for facing right:
        rightImage = new GreenfootImage("wombat.png");
        rightImage.scale(40,31); // make it a bit smaller
        
        leftImage = new GreenfootImage(rightImage); // create a copy of rightImage
        leftImage.mirrorHorizontally(); // mirror it
        
        setImage(rightImage); // begin with facing right
    }
    
    /**
     * Jump if we see platforms, move to left
     * and right, 'die' when we hit the bottom.
     */
    public void act() 
    {
        move();
        handleGravity();
        checkEdges();
    }
    
    /**
     * If you press left, move left.
     * If you press right, move right.
     */
    private void move()
    {
        if (Greenfoot.isKeyDown("left")) {
            setLocation(getX() -5, getY());
            setImage(leftImage); // face left
        }
        if (Greenfoot.isKeyDown("right")) {
            setLocation(getX() +5, getY());
            setImage(rightImage); // face right
        }
    }
    
    /**
     * Jumps if we see a platform, else we will fall and accelerate.
     */
    private void handleGravity()
    {
        setLocation(getX(), getY() -gravity);
        gravity--; // accelerate
        
        // jump when:
        if (canSee(Platform.class) // we see a platform  
        && (gravity <= 0 // we didn't jump very recently
        || getY() > 0)) { // don't jump to high
            gravity = 20;
        }
    }
    
    /**
     * Moves from the left side to the right side,
     * and from the right side to the left side.
     * Stop when we hit the bottom
     */
    private void checkEdges()
    {
        if (atWhichEdge() == 0) { // if at left edge, set location to the right edge.
            setLocation(getWorld().getWidth() -2, getY());
        } else if (atWhichEdge() == 1) { // if at right edge, set location to the left edge.
            setLocation(2, getY());
        } else if(atWhichEdge() == 2 // if at the top
        && getY() <= 20) { // and if we're high enough:
            JumpWorld w = (JumpWorld) getWorld();
            w.movePlatforms(8); // move platforms down
            w.scroll(2); // scroll the background
        } else if (atWhichEdge() == 3) { // if at the bottom:
            Greenfoot.stop(); // you would go game over here
            // in the origional game, I accessed the world and called
            // the 'gameOver' method, which would show the scoreboard.
        }
    }
    
    /**
     * Return true if we can see an object of class 'clss' right where we are. 
     * False if there is no object here.
     * @param clss The class you want to check you see it.
     */
    private boolean canSee(Class clss)
    {
        // this is pretty basic, so I'm not going to explain this
        return getOneIntersectingObject(clss) != null;
    }
    
    /**
     * Returns a number of which side of the world you're on.
     * If it's 0, left edge.
     * If it's 1, right edge.
     * If it's 2, top edge.
     * If it's 3, bottom edge.
     * If it's 4, you're at no edge.
     */
    private int atWhichEdge()
    {
        if (getY() >= getWorld().getHeight() -getImage().getHeight() /2 -1) { // bottom
            return 3; // the bottom is the most important (you can be at
            // the left edge and the bottom at the same time, but you
            // still want to die!), so we check that first
        } else if(getX() <= 1) { // right edge
            return 0;
        } else if (getX() >= getWorld().getWidth() -1) { // left edge
            return 1;
        } else if (getY() <= getImage().getHeight() /2 -1) { // top
            return 2;
        } else { // none of above: no edge
            return 4;
        }
    }
}